package main.mavenchess;


import javafx.application.Application;
import javafx.beans.property.DoubleProperty;
import javafx.beans.property.SimpleDoubleProperty;
import javafx.scene.*;
import javafx.scene.effect.Bloom;
import javafx.scene.image.Image;
import javafx.scene.input.KeyEvent;
import javafx.scene.input.MouseEvent;
import javafx.scene.paint.Color;
import javafx.scene.paint.PhongMaterial;
import javafx.scene.shape.Box;
import javafx.scene.shape.DrawMode;
import javafx.scene.shape.TriangleMesh;
import javafx.scene.transform.Rotate;
import javafx.scene.transform.Transform;
import javafx.scene.transform.Translate;
import javafx.stage.Stage;
import javafx.fxml.FXMLLoader;
import javafx.scene.shape.MeshView;

import java.io.File;
import java.io.FileNotFoundException;
import java.net.URL;
import java.util.Scanner;


public class HelloApplication extends Application {
	// Size
	private static final int BOARD_SIZE = 16, CENTERED_ORIGIN = -BOARD_SIZE*7/2;
	// Camera
	private static double anchorX, anchorY, anchorAngleX, anchorAngleY;
	private static DoubleProperty angleX, angleY;
	private static PerspectiveCamera camera;
	private static Group camera_group; // used to rotate camera from 0,0,0 and not from camera x,y,z
	// Material
	private static PhongMaterial[] material;
	private static final int MATERIAL_BLACK = 0, MATERIAL_WHITE = 1, MATERIAL_SELECTED = 2, MATERIAL_MOVABLE = 3,
			MATERIAL_GRAY = 4, MATERIAL_WOOD = 5;
	// Scene
	private static Scene scene;
	private static Group group;
	private static Box[] board;

	@Override
	public void start(Stage stage) {

		stage.setMaximized(true); // maximized window
		stage.setTitle("Chess3D - Tuna Acikbas & Olivier Pillods");
		group = new Group(); // will contain all the elements "to show" on scene

		/*group.setCache(true);
		Bloom bloom = new Bloom();
		bloom.setThreshold(1.0);
		group.setEffect(bloom);*/

		initMaterial(); // prepare all the 3D materials (shaders)
		initCamera(1, 1000, 70); // create a camera and some settings
		initScene(SceneAntialiasing.BALANCED); // create a scene, a background, and links camera to scene
		initMouseControl(camera_group, scene); // camera control with mouse

		initChessBoard(); // create a chessboard of 8x8 and some click events


		TriangleMesh mesh = new TriangleMesh();
		mesh.getTexCoords().addAll(0,0,0,0, 0, 0);
		mesh.getPoints().addAll(0,0,0);
		try {
			URL url = this.getClass().getResource("/pion.vertice");
			File myObj = new File(url.getPath().replaceAll("%20", " "));
			Scanner myReader = new Scanner(myObj);
			while (myReader.hasNextLine()) {
				String data = myReader.nextLine();
				String[] data2 = data.split(",");
				mesh.getPoints().addAll(
					Float.parseFloat(data2[0]),
					Float.parseFloat(data2[1]),
					Float.parseFloat(data2[2])
				);
			}
			myReader.close();
			URL url2 = this.getClass().getResource("/pion.mesh");
			File myObj2 = new File(url2.getPath().replaceAll("%20", " "));
			Scanner myReader2 = new Scanner(myObj2);
			while (myReader2.hasNextLine()) {
				String data3 = myReader2.nextLine();
				String[] data4 = data3.split(",");
				mesh.getFaces().addAll(
					Integer.parseInt(data4[0]), 0,
					Integer.parseInt(data4[1]), 0,
					Integer.parseInt(data4[2]), 0
				);
			}
			myReader2.close();
		} catch (FileNotFoundException e) {
			System.out.println("An error occurred.");
			e.printStackTrace();
		}

		//System.out.println(mesh.getFaces());

		//PhongMaterial materialP = new PhongMaterial();
		//materialP.setDiffuseMap(new Image("/ScreenShot177.png"));

		MeshView pyramid = new MeshView(mesh);
		pyramid.setDrawMode(DrawMode.FILL);
		pyramid.setMaterial(material[MATERIAL_WHITE]);
		pyramid.translateYProperty().set((float)-12);
		pyramid.translateXProperty().set((float)8);
		pyramid.translateZProperty().set((float)8);
		int scale = 16;
		pyramid.setScaleX(scale);
		pyramid.setScaleY(-scale);
		pyramid.setScaleZ(scale);


		group.getChildren().addAll(pyramid);
		AmbientLight ambient = new AmbientLight(Color.rgb(35, 35, 35));
		float intensity = 0.5f;
		PointLight[] light = {
			new PointLight(Color.rgb(191, 174, 148)),
			new PointLight(Color.rgb(191, 174, 148)),
			new PointLight(Color.rgb(191, 174, 148))
		};
		light[0].getTransforms().add(new Translate(-100, -50, 100));
		light[1].getTransforms().add(new Translate(0, -60, -100));
		light[2].getTransforms().add(new Translate(100, -50, 100));
		SpotLight spot = new SpotLight(Color.rgb(255,0,0));


		group.getChildren().addAll(light[0], light[1], light[2], ambient, spot);

		int table_size = BOARD_SIZE*12;
		Box table = new Box(table_size*2, BOARD_SIZE, table_size);
		table.translateYProperty().set(BOARD_SIZE/2);
		table.setMaterial(material[MATERIAL_WOOD]);
		group.getChildren().add(table);

		//FXMLLoader loader = new FXMLLoader(HelloApplication.class.getResource("chess_export2.fxml"));
		//VBox vbox = loader.<VBox>load();

		/*
		ModelImporter objImporter = new ObjModelImporter();
		try{
			URL modelUrl = this.getClass().getResource("assets/David.obj");
			objImporter.read(modelUrl);
		}catch(ImportException e){
			System.out.println("Pas d'url");
			System.out.println(e.getMessage());
		}
		MeshView[] meshViews = objImporter.getImport();
		Group David = new Group(meshViews);*/

		// manual zoom (Z and S)
		stage.addEventHandler(KeyEvent.KEY_PRESSED, event -> {
			switch (event.getCode()) {
				case Z -> camera.translateZProperty().set(camera.getTranslateZ() + 10);
				case S -> camera.translateZProperty().set(camera.getTranslateZ() - 10);
			}
		});

		// start
		stage.setScene(scene);
		stage.show();
	}

	public static void main(String[] args) {
		launch();
	}

	private void initMaterial() {
		material = new PhongMaterial[6];
		material[MATERIAL_BLACK] = new PhongMaterial();
		material[MATERIAL_BLACK].setDiffuseColor(Color.BLACK);
		material[MATERIAL_BLACK].setSpecularColor(Color.GRAY);
		material[MATERIAL_WHITE] = new PhongMaterial();
		material[MATERIAL_WHITE].setDiffuseColor(Color.WHITE);
		material[MATERIAL_WHITE].setSpecularColor(Color.GRAY);
		material[MATERIAL_SELECTED] = new PhongMaterial();
		material[MATERIAL_SELECTED].setDiffuseColor(Color.CYAN);
		material[MATERIAL_SELECTED].setSpecularColor(Color.GRAY);
		material[MATERIAL_MOVABLE] = new PhongMaterial();
		material[MATERIAL_MOVABLE].setDiffuseColor(Color.GREEN);
		material[MATERIAL_MOVABLE].setSpecularColor(Color.GRAY);
		material[MATERIAL_GRAY] = new PhongMaterial();
		material[MATERIAL_GRAY].setDiffuseColor(Color.DARKGRAY);
		material[MATERIAL_GRAY].setSpecularColor(Color.GRAY);
		material[MATERIAL_WOOD] = new PhongMaterial();
		material[MATERIAL_WOOD].setDiffuseMap(new Image("/wood.jpg"));
		material[MATERIAL_WOOD].setSpecularColor(Color.GRAY);
		//material.setDiffuseMap(new Image("/ScreenShot177.png"));
	}

	private void initCamera(int nearClip, int farClip, int fov) {
		camera = new PerspectiveCamera(true);
		camera.setFieldOfView(fov);
		camera.translateXProperty().set(0);
		camera.translateYProperty().set(0);
		camera.translateZProperty().set((float)-130);
		camera.setNearClip(nearClip);
		camera.setFarClip(farClip);
		camera_group = new Group(camera);
		camera_group.translateYProperty().set(20);
		// camera controls for user (with mouse)
		angleX = new SimpleDoubleProperty(-45);
		angleY = new SimpleDoubleProperty(8);
	}

	private void initScene(SceneAntialiasing antialiasing) {
		scene = new Scene(group, 800, 600, true, antialiasing);
		scene.setFill(Color.rgb(50, 50, 50));
		scene.setCamera(camera);
	}

	private void initChessBoard() {
		board = new Box[8*8];
		for(int j = 0 ; j < 8 ; j++) {
			for(int i = 0 ; i < 8 ; i++) {
				int n = j*8+i;
				board[n] = new Box(BOARD_SIZE, BOARD_SIZE/2, BOARD_SIZE);
				board[n].translateXProperty().set(i*BOARD_SIZE + CENTERED_ORIGIN);
				board[n].translateZProperty().set(j*BOARD_SIZE + CENTERED_ORIGIN);
				board[n].setMaterial(material[(i+j)%2]);
				// board click event
				board[n].addEventHandler(MouseEvent.MOUSE_CLICKED, event -> {
					for(int j2 = 0 ; j2 < 8 ; j2++) {
						for (int i2 = 0; i2 < 8; i2++) {
							int n2 = j2 * 8 + i2;
							board[n2].setMaterial(material[(i2+j2)%2]);
						}
					}
					board[n].setMaterial(material[MATERIAL_SELECTED]);
				});
			}
		}
		group.getChildren().addAll(board);
	}

	private void initMouseControl(Group group, Scene scene) {
		Rotate xRotate;
		Rotate yRotate;
		group.getTransforms().addAll(
				yRotate = new Rotate(0, Rotate.Y_AXIS),
				xRotate = new Rotate(-25, Rotate.X_AXIS)
		);
		xRotate.angleProperty().bind(angleX);
		yRotate.angleProperty().bind(angleY);

		scene.setOnMousePressed(event -> {
			anchorX = event.getSceneX();
			anchorY = event.getSceneY();
			anchorAngleX = angleX.get();
			anchorAngleY = angleY.get();

		});

		int min = -90;
		int max = -20;

		scene.setOnMouseDragged(event -> {
			angleX.set(anchorAngleX + (anchorY - event.getSceneY()));
			angleY.set(anchorAngleY - (anchorX - event.getSceneX()));
			if(angleX.get() > max) {
				angleX.set(max);
				anchorY = event.getSceneY();
				anchorAngleX = angleX.get();
			}
			if(angleX.get() < min) {
				angleX.set(min);
				anchorY = event.getSceneY();
				anchorAngleX = angleX.get();
			}
		});
	}
}